import "Turbine";
import "Turbine.Debug";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Lotro";
import "DhorPlugins.Extensions";
import "DhorPlugins.Utils.BitOps";


--[[ This is the main window for the Travel Window UI	                      ]]--
--[[ It handles the loading and saving the settings,	                      ]]--
--[[ creating the lists of travel shortcuts, creates	                      ]]--
--[[ and handles the context menu.						                         ]]--


MainWindow = class( Turbine.UI.Extensions.SimpleWindow );

function MainWindow:Constructor()
	Turbine.UI.Extensions.SimpleWindow.Constructor( self );	
	
   -- get the player class and race
   player = Turbine.Gameplay.LocalPlayer.GetInstance();
   playerClass = player:GetClass();
   
   -- quit if the class is not a hunter                        
   if(playerClass ~= Turbine.Gameplay.Class.Hunter) then
      Turbine.Shell.WriteLine("This plugin will only work for hunters.");
      return;
   end
   
   -- create the settings	
	settings = { };
   settingStrings = { };  	
	self.minWidth = 21;
	self.minHeight = 21;
	self.options = nil;
	
	-- load the player saved settings
	self:LoadSettings();

	-- configure the main window
	self:SetPosition(settings.positionX,settings.positionY);	
	self:SetSize( settings.width, settings.height );
	self:SetOpacity( settings.mainMinOpacity );
	self:SetZOrder(1);
	self:SetBackColor(settings.backColor);
	self:SetVisible(true); 
   
   -- create a control for resizing the window
   self.sizeControl = Turbine.UI.Control();
   self.sizeControl:SetParent(self);
   self.sizeControl:SetZOrder(100);
   self.sizeControl:SetSize(20,20);
   self.sizeControl:SetBackColorBlendMode(Turbine.UI.BlendMode.Screen)
   self.sizeControl:SetBackColor(Turbine.UI.Color(0.01,1,1,1));
   self.sizeControl:SetPosition(self:GetWidth() - self.sizeControl:GetWidth(),
                            self:GetHeight() - self.sizeControl:GetHeight());   
   
   -- add the labels for the focus level   
   self.labels = { };
   for i = 1,9,1 do
      self.labels[i] = Turbine.UI.Label();		
		self.labels[i]:SetParent( self );
		self.labels[i]:SetBackColor(settings.focusColor);
		self.labels[i]:SetZOrder(50);
		self.labels[i]:SetMouseVisible(false);
   end
   
   -- add the label for the focus number
   
   self.numberLabel = Turbine.UI.Label();		
	self.numberLabel:SetParent( self );
	self.numberLabel:SetSize(19,19);
   self.numberLabel:SetFont(Turbine.UI.Lotro.Font.TrajanPro19);	
   self.numberLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	self.numberLabel:SetZOrder(51);
	self.numberLabel:SetMouseVisible(false);  
   
	-- create a single context menu to use on all panels
	menu = Turbine.UI.ContextMenu(self);
   self:SetupMenu();
		
	-- hunter attribute
	self.hunterFocus = player:GetClassAttributes();
	self.focusLogging = false;
	self.loggingStartTime = nil;
	self.lastCombatLog = { };
	
   -- timers for events
	self.lastTime = 0;
   self.lastMove = 0;
   
   -- track the hidden state of the UI
   self.hidden = false;
   
   -- setup the labels for the focus
   self:UpdateLabels();
   self:UpdateColors();
   self:UpdateFocus(self.hunterFocus:GetFocus());     
   
--[[ Event Handlers ]]--

	-- make sure we listen for updates
	self:SetWantsUpdates(true);
	self:SetWantsKeyEvents(true);
	
   local isMoving = false;
	local x = 0;
	local y = 0;

   -- handle window mouse down event (not quickslot or button events)
	self.MouseDown = function(sender, args )
	   -- if its the left button, prepare to move the window by setting
	   -- the flag, and storing the current mouse position
		if (args.Button == Turbine.UI.MouseButton.Left  and settings.lock == 0) then
			isMoving = true;
			x = args.X;
			y = args.Y;
		elseif(args.Button == Turbine.UI.MouseButton.Right and player:IsInCombat() == false) then
		   menu:ShowMenu();
		end
	end

   -- handle window mouse up event
	self.MouseUp = function(sender, args )
	   -- check if its the left mouse button, otherwise ignore
		if (args.Button == Turbine.UI.MouseButton.Left) then
		
		   -- clear the flag
			isMoving = false;

         -- save the current position settings
			local one,two = self:GetPosition();
			settings.positionX = one;
			settings.positionY = two;
			self:SaveSettings();
		end
	end

   -- handle window mouse move event
	self.MouseMove = function(sender, args )
	   -- if the moving flag is set, move the window
		if (isMoving) then
		   -- get the old position of the window
			local oldX,oldY = self:GetPosition();
			
			-- update the window position based on the previous mouse location,
			-- previous window position, and the current mouse position
			self:SetPosition(oldX + args.X - x,oldY + args.Y - y);	
		end
	end
   	
	-- check if our position has changed, and save the settings if so
	self.PositionChanged = function(sender, args )
		one,two = self:GetPosition();
		settings.positionX = one;
		settings.positionY = two;
		self:SaveSettings();
	end
	
   -- manage hiding the UI  with F12
   self.KeyDown = function( sender, args)
      if(args.Action == 268435635) then
         if(self.hidden == false) then
             self.currentVisState = self:IsVisible();
             self.hidden = true;
             self:SetVisible(false);             
         else
             self.hidden = false;
             self:SetVisible(self.currentVisState);
         end
      end
   end
   
   -- check if the player's focus level changes
   self.hunterFocus.FocusChanged = function(sender, args)
   
      self:UpdateFocus(self.hunterFocus:GetFocus());
      
      if(self.focusLogging) then
         table.insert(self.lastCombatLog,{Turbine.Engine.GetGameTime() - self.loggingStartTime,self.hunterFocus:GetFocus()});
      end
   end
   
   self.hunterFocus.StanceChanged = function(sender, args)
      self:UpdateColors();
   end
   
   -- check if the combat state of the player changed, to adjust the opacity
   -- of the window
   inCombatChangedHandler = function(sender, args)

      if(player:IsInCombat()) then
         self:SetOpacity(settings.mainMaxOpacity)
           self.focusLogging = true;
	        self.loggingStartTime = Turbine.Engine.GetGameTime();
	        self.lastCombatLog = { };
	        
	        table.insert(self.lastCombatLog,{0,self.hunterFocus:GetFocus()});
      else
         self.focusLogging = false;
         self:SetOpacity(settings.mainMinOpacity)
	   end
   end
   AddCallback(player, "InCombatChanged", inCombatChangedHandler);

	-- change the background color of the resize button on entry to help
	-- show exactly where the button is
	self.sizeControl.MouseEnter = function(sender,args)
     if(settings.lock == 0) then     	  
      self.sizeControl:SetBackColor(Turbine.UI.Color(0.9,1,1,1));
     end
	end
	
	-- return background color to normal on mouse leave
	self.sizeControl.MouseLeave = function(sender,args)
	   self.sizeControl:SetBackColor(Turbine.UI.Color(0.01,1,1,1));
	end
	
	-- check if someone is going to start dragging the resize control
	self.sizeControl.MouseDown = function(sender,args)
	
	  -- check if resize has been disabled
	  if(self.disableResize or settings.lock == 1) then
	     return;
	  end  
	
	  -- set start location of resize and enable dragging
	  sender.dragStartX = args.X;
	  sender.dragStartY = args.Y;
	  sender.dragging = true;
	end
	
	self.sizeControl.MouseUp = function(sender,args)
	  -- disable dragging, then update settings
	  sender.dragging = false;	  
	  
	  settings.width = self:GetWidth();
	  settings.height = self:GetHeight();	  
	  self:SaveSettings();
	end
	
	-- perform the resize
	self.sizeControl.MouseMove = function(sender,args)	
	  	 local width, height = self:GetSize();
      
         -- if resize dragging, then resize the window 		
		 if ( sender.dragging ) then
			   self:SetSize( width + ( args.X - sender.dragStartX ), 
                        height + ( args.Y - sender.dragStartY ) );
		 end
		 
		 -- update the size of the window
		 self:UpdateSize();      
	end
      
   Plugins["HunterFocus"].Load = function(sender,args)
      Turbine.Shell.WriteLine("Hunter Focus version " .. Plugins["HunterFocus"]:GetVersion() .. " loaded"); 
   end     
                          
end

function MainWindow:UpdateSize()
         -- check that the window is not smaller than min width	
		 if (self:GetWidth() < self.minWidth) then
		       self:SetWidth(self.minWidth);
		 end
		
		 -- check that the window is not smaller than min height
		 if (self:GetHeight() < self.minHeight) then
		       self:SetHeight(self.minHeight);
       end
      
       -- set the new location of the resize control
		 self.sizeControl:SetPosition( 
                           self:GetWidth() - self.sizeControl:GetWidth(), 
                           self:GetHeight() - self.sizeControl:GetHeight() );
      
                           
       self:SaveSettings();
       self:UpdateLabels();
end

function MainWindow:SetupMenu()
	self.options = Turbine.UI.MenuItem(optionsString);
	self.focusLog = Turbine.UI.MenuItem(logString);
	menuItems = menu:GetItems();
	menuItems:Add(self.options);
	menuItems:Add(self.focusLog);
	
	-- open the option window when clicked
	menuItems:Get(1).Click = function(sender, args)
	   self:OpenOptions();
	end
	
	menuItems:Get(2).Click = function(sender,args)
	   self:ShowLog();
	end
end  

function MainWindow:LoadSettings()
	-- load the self.settings.  If a value is not available, set a default value
	settingStrings = Turbine.PluginData.Load( Turbine.DataScope.Character, 
                                                "HunterFocusSettings" );

	if ( type( settingStrings ) ~= "table" ) then
		settingStrings = { };
	end
	
	if ( not settingStrings.lock or settingStrings.lock == "nil") then
	   settingStrings.lock = tostring(0);
	end
	
	if ( not settingStrings.width or settingStrings.width == "nil") then
	   settingStrings.width = tostring(200);
	end
	
	if (not settingStrings.height or settingStrings.height == "nil") then
	   settingStrings.height = tostring(50);
	end

	if ( not settingStrings.positionX or settingStrings.positionX == "nil" ) then
		settingStrings.positionX = tostring(200); 
	end

	if ( not settingStrings.positionY or settingStrings.positionY == "nil" ) then
		settingStrings.positionY = tostring(200);
	end
	
	if ( not settingStrings.hideOnStart or settingStrings.hideOnStart == "nil" ) then
	   settingStrings.hideOnStart = tostring(0);
	end
   	
	if ( not settingStrings.hideOnCombat or settingStrings.hideOnCombat == "nil" ) then
	   settingStrings.hideOnCombat = tostring(0);
	end
	
	if ( not settingStrings.horizontal or settingStrings.horizontal == "nil" ) then
	   settingStrings.horizontal = tostring(1);
	end

	if ( not settingStrings.mainMaxOpacity or settingStrings.mainMaxOpacity == "nil" ) then
	   settingStrings.mainMaxOpacity = tostring(1);
	end
	
	if ( not settingStrings.mainMinOpacity or settingStrings.mainMinOpacity == "nil" ) then
	  settingStrings.mainMinOpacity = tostring(0.5);
	end
	
	if ( not settingStrings.showBars or settingStrings.showBars == "nil" ) then
	  settingStrings.showBars = tostring(1);
	end
	
	if ( not settingStrings.showNumber or settingStrings.showNumber == "nil" ) then
	  settingStrings.showNumber = tostring(1);
	end
	
	if ( not settingStrings.focusColor or settingStrings.focusColor == "nil" ) then
	  settings.focusColor = Turbine.UI.Color(1,0.9,0.1,0.1);
	end
	
	if ( not settingStrings.focusSColor or settingStrings.focusSColor == "nil" ) then
	  settings.focusSColor = Turbine.UI.Color(1,0.9,0.1,0.1);
	end
	
	if ( not settingStrings.focusPColor or settingStrings.focusPColor == "nil" ) then
	  settings.focusPColor = Turbine.UI.Color(1,0.9,0.1,0.1);
	end
	
	if ( not settingStrings.focusEColor or settingStrings.focusEColor == "nil" ) then
	  settings.focusEColor = Turbine.UI.Color(1,0.9,0.1,0.1);
	end
	
	if ( not settingStrings.backColor or settingStrings.backColor == "nil" ) then
	  settings.backColor = Turbine.UI.Color(1,1,1,1);
	end
	
	if ( not settingStrings.numberColor or settingStrings.numberColor == "nil" ) then
	  settings.numberColor = Turbine.UI.Color(1,1,1,1);
	end

	-- convert from strings if necessary 
   for i,v in pairs(settingStrings) do
      
      if(type(v) == "string") then
         settings[i] = tonumber(v);
      elseif(type(v) == "table") then
         self:convertColorToNumbers(i,v);
      else
         settings[i] = v;
      end
   end

end

function MainWindow:SaveSettings()
	-- save the settings
	
	settingStrings = { };
	
	for i,v in pairs(settings) do
	   if(type(v) ~= "table") then
	      settingStrings[i] = tostring(v);
	   else
	      self:convertColorToStrings(i,v);
	   end
	end
	
	Turbine.PluginData.Save( Turbine.DataScope.Character,
                                  "HunterFocusSettings", settingStrings );
end

function MainWindow:convertColorToStrings(index, value)

   tempTable = { };

   for i,v in pairs(value) do
      if(type(v) == "number") then
         tempTable[i] = tostring(v);
      else
         tempTable[i] = v;
      end  
   end

   settingStrings[index] = tempTable;

end

function MainWindow:convertColorToNumbers(index, value)

   tempColor = Turbine.UI.Color(0,0,0,0);

   for i,v in pairs(value) do   
     if(type(v) == "string") then
        tempColor[i] = tonumber(v);
     else
        tempColor[i] = v;
     end
   end

   settings[index] = tempColor;

end

function MainWindow:UpdateLabels()

   -- loop through the labels and adjust the size and position of each
   for i = 1,9,1 do
      if(settings.horizontal == 1) then
         local labelWidth = (self:GetWidth() - 10)/9;    
         self.labels[i]:SetSize(labelWidth,self:GetHeight()-2);
         self.labels[i]:SetPosition((i-1) * labelWidth + i, 1);
      else
         local labelHeight = (self:GetHeight() - 10)/9;    
         self.labels[i]:SetSize(self:GetWidth() - 2,labelHeight);
         self.labels[i]:SetPosition(1,self:GetHeight() - ((i-1) * labelHeight + i) - labelHeight);
      end 
   end                                                 
   
   -- set the visibility and size of the number label
   self.numberLabel:SetVisible(settings.showNumber == 1);
   self.numberLabel:SetSize(19,19);   
   self.numberLabel:SetPosition(self:GetWidth() / 2 - 9,self:GetHeight() / 2 - 9);
   
   self:SetBackColor(settings.backColor);       
end

function MainWindow:UpdateFocus()

   level = self.hunterFocus:GetFocus();

   -- loop through all the labels and set the visibility based on the
   -- current level of focus   
   for i = 1,9,1 do
     if(i <= level and (settings.showBars == 1)) then
       self.labels[i]:SetVisible(true);
     else
       self.labels[i]:SetVisible(false);
     end
   end   
   
   -- update the number label
   self.numberLabel:SetText(level);
   
end

function MainWindow:UpdateColors()

   -- default color
   colorToUse = Turbine.UI.Color(1,1,1,1);
   
   -- select color based on stance
   if(self.hunterFocus:GetStance() == Turbine.Gameplay.Attributes.HunterStance.None) then
      colorToUse = settings.focusColor;        	
   elseif(self.hunterFocus:GetStance() == Turbine.Gameplay.Attributes.HunterStance.Strength) then
      colorToUse = settings.focusSColor;        	
   elseif(self.hunterFocus:GetStance() == Turbine.Gameplay.Attributes.HunterStance.Precision) then
      colorToUse = settings.focusPColor;        	
   elseif(self.hunterFocus:GetStance() == Turbine.Gameplay.Attributes.HunterStance.Endurance) then
      colorToUse = settings.focusEColor;        	
   end
   
   -- set the color to all the labels
   for i = 1,9,1 do
      self.labels[i]:SetBackColor(colorToUse);
   end
   
   self.numberLabel:SetBackColor(colorToUse);
   self.numberLabel:SetForeColor(settings.numberColor);
   
end   

-- for debug purposes
function MainWindow:DoDump()
	Turbine.Debug.Table.Dump(DhorPlugins.HunterFocus);
end

-- simple function to open the options window
function MainWindow:OpenOptions()
    self.options = DhorPlugins.HunterFocus.OptionsWindow(self);
end

-- function to close the options window if it exists
function MainWindow:CloseOptions()
   if(self.options ~= nil) then
      self.options:SetVisible(false);
   end
   self.options = nil;
end

function MainWindow:UpdateOpacity()
   self:SetOpacity(settings.mainMinOpacity);
end

function MainWindow:LockWindow()
   Turbine.Shell.WriteLine("Hunter focus window has been locked");
   settings.lock = 1;
   self:SaveSettings();
   
end 

function MainWindow:UnlockWindow()
   Turbine.Shell.WriteLine("Hunter focus window has been unlocked");
   settings.lock = 0;
   self:SaveSettings();
end

function MainWindow:ShowLog()
   self.logWindow = DhorPlugins.HunterFocus.LogWindow(self,self.lastCombatLog);
end

-- function to close the options window if it exists
function MainWindow:CloseLog()
   if(self.logWindow ~= nil) then
      self.logWindow:SetVisible(false);
   end
   self.logWindow = nil;
end

function AddCallback(object, event, callback)
	if (object[event] == nil) then
        	object[event] = callback;
	else
        	if (type(object[event]) == "table") then
           		table.insert(object[event], callback);
	        else
        		object[event] = {object[event], callback};
	        end
	end
	return callback;
end